@@ -404,12 +404,13 @@ config_tag_test(void)
 	
 #include "util/rtt.h"
 #include "util/timehist.h"
+#include "iterator/iterator.h"
 #include "libunbound/unbound.h"
 /** test RTT code */
 static void
 rtt_test(void)
 {
-	int init = 376;
+	int init = UNKNOWN_SERVER_NICENESS;
 	int i;
 	struct rtt_info r;
 	unit_show_func("util/rtt.c", "rtt_timeout");
@@ -905,6 +906,9 @@ main(int argc, char* argv[])
 #ifdef CLIENT_SUBNET
 	ecs_test();
 #endif /* CLIENT_SUBNET */
+	if(log_get_lock()) {
+		lock_quick_destroy((lock_quick_type*)log_get_lock());
+	}
 	checklock_stop();
 	printf("%d checks ok.\n", testcount);
 #ifdef HAVE_SSL
