@@ -156,6 +156,12 @@ struct reply_info {
 	 */
 	time_t prefetch_ttl;
 
+	/** 
+	 * Reply TTL extended with serve expired TTL, to limit time to serve
+	 * expired message.
+	 */
+	time_t serve_expired_ttl;
+
 	/**
 	 * The security status from DNSSEC validation of this message.
 	 */
@@ -222,6 +228,7 @@ struct msgreply_entry {
  * @param qd: qd count
  * @param ttl: TTL of replyinfo
  * @param prettl: prefetch ttl
+ * @param expttl: serve expired ttl
  * @param an: an count
  * @param ns: ns count
  * @param ar: ar count
@@ -232,8 +239,8 @@ struct msgreply_entry {
  */
 struct reply_info*
 construct_reply_info_base(struct regional* region, uint16_t flags, size_t qd,
-		time_t ttl, time_t prettl, size_t an, size_t ns, size_t ar,
-		size_t total, enum sec_status sec);
+		time_t ttl, time_t prettl, time_t expttl, size_t an, size_t ns,
+		size_t ar, size_t total, enum sec_status sec);
 
 /** 
  * Parse wire query into a queryinfo structure, return 0 on parse error. 
@@ -285,6 +292,10 @@ int parse_create_msg(struct sldns_buffer* pkt, struct msg_parse* msg,
         struct alloc_cache* alloc, struct query_info* qinf,
 	struct reply_info** rep, struct regional* region);
 
+/** get msg reply struct (in temp region) */
+struct reply_info* parse_reply_in_temp_region(struct sldns_buffer* pkt,
+	struct regional* region, struct query_info* qi);
+
 /**
  * Sorts the ref array.
  * @param rep: reply info. rrsets must be filled in.
@@ -541,12 +552,13 @@ struct edns_option* edns_opt_list_find(struct edns_option* list, uint16_t code);
  * @param rep: Reply info. Could be NULL.
  * @param rcode: return code.
  * @param edns: edns data of the reply.
+ * @param repinfo: comm_reply. NULL.
  * @param region: region to store data.
  * @return false on failure (a callback function returned an error).
  */
 int inplace_cb_reply_call(struct module_env* env, struct query_info* qinfo,
 	struct module_qstate* qstate, struct reply_info* rep, int rcode,
-	struct edns_data* edns, struct regional* region);
+	struct edns_data* edns, struct comm_reply* repinfo, struct regional* region);
 
 /**
  * Call the registered functions in the inplace_cb_reply_cache linked list.
@@ -557,13 +569,14 @@ int inplace_cb_reply_call(struct module_env* env, struct query_info* qinfo,
  * @param rep: Reply info.
  * @param rcode: return code.
  * @param edns: edns data of the reply. Edns input can be found here.
+ * @param repinfo: comm_reply. Reply information for a communication point.
  * @param region: region to store data.
  * @return false on failure (a callback function returned an error).
  */
 int inplace_cb_reply_cache_call(struct module_env* env,
 	struct query_info* qinfo, struct module_qstate* qstate,
 	struct reply_info* rep, int rcode, struct edns_data* edns,
-	struct regional* region);
+	struct comm_reply* repinfo, struct regional* region);
 
 /**
  * Call the registered functions in the inplace_cb_reply_local linked list.
@@ -574,13 +587,14 @@ int inplace_cb_reply_cache_call(struct module_env* env,
  * @param rep: Reply info.
  * @param rcode: return code.
  * @param edns: edns data of the reply. Edns input can be found here.
+ * @param repinfo: comm_reply. Reply information for a communication point.
  * @param region: region to store data.
  * @return false on failure (a callback function returned an error).
  */
 int inplace_cb_reply_local_call(struct module_env* env,
 	struct query_info* qinfo, struct module_qstate* qstate,
 	struct reply_info* rep, int rcode, struct edns_data* edns,
-	struct regional* region);
+	struct comm_reply* repinfo, struct regional* region);
 
 /**
  * Call the registered functions in the inplace_cb_reply linked list.
@@ -592,13 +606,14 @@ int inplace_cb_reply_local_call(struct module_env* env,
  * @param rcode: return code. LDNS_RCODE_SERVFAIL.
  * @param edns: edns data of the reply. Edns input can be found here if qstate
  *	is NULL.
+ * @param repinfo: comm_reply. Reply information for a communication point.
  * @param region: region to store data.
  * @return false on failure (a callback function returned an error).
  */
 int inplace_cb_reply_servfail_call(struct module_env* env,
 	struct query_info* qinfo, struct module_qstate* qstate,
 	struct reply_info* rep, int rcode, struct edns_data* edns,
-	struct regional* region);
+	struct comm_reply* repinfo, struct regional* region);
 
 /**
  * Call the registered functions in the inplace_cb_query linked list.
