@@ -166,6 +166,9 @@ struct query_info;
 struct edns_data;
 struct regional;
 struct worker;
+struct comm_base;
+struct auth_zones;
+struct outside_network;
 struct module_qstate;
 struct ub_randstate;
 struct mesh_area;
@@ -233,8 +236,8 @@ struct inplace_cb {
 
 /**
  * Inplace callback function called before replying.
- * Called as func(edns, qstate, opt_list_out, qinfo, reply_info, rcode,
- *                region, python_callback)
+ * Called as func(qinfo, qstate, rep, rcode, edns, opt_list_out, repinfo,
+ *                region, id, python_callback)
  * Where:
  *	qinfo: the query info.
  *	qstate: the module state. NULL when calling before the query reaches the
@@ -244,18 +247,23 @@ struct inplace_cb {
  *	edns: the edns_data of the reply. When qstate is NULL, it is also used as
  *		the edns input.
  *	opt_list_out: the edns options list for the reply.
+ *	repinfo: reply information for a communication point. NULL when calling
+ *		during the mesh states; the same could be found from
+ *		qstate->mesh_info->reply_list.
  *	region: region to store data.
+ *	id: module id.
  *	python_callback: only used for registering a python callback function.
  */
 typedef int inplace_cb_reply_func_type(struct query_info* qinfo,
 	struct module_qstate* qstate, struct reply_info* rep, int rcode,
-	struct edns_data* edns, struct edns_option** opt_list_out, 
-	struct regional* region, int id, void* callback);
+	struct edns_data* edns, struct edns_option** opt_list_out,
+	struct comm_reply* repinfo, struct regional* region, int id,
+	void* callback);
 
 /**
  * Inplace callback function called before sending the query to a nameserver.
  * Called as func(qinfo, flags, qstate, addr, addrlen, zone, zonelen, region,
- *                python_callback)
+ *                id, python_callback)
  * Where:
  *	qinfo: query info.
  *	flags: flags of the query.
@@ -267,6 +275,7 @@ typedef int inplace_cb_reply_func_type(struct query_info* qinfo,
  *		authoritative.
  *	zonelen: length of zone.
  *	region: region to store data.
+ *	id: module id.
  *	python_callback: only used for registering a python callback function.
  */
 typedef int inplace_cb_query_func_type(struct query_info* qinfo, uint16_t flags,
@@ -276,10 +285,10 @@ typedef int inplace_cb_query_func_type(struct query_info* qinfo, uint16_t flags,
 
 /**
  * Inplace callback function called after parsing edns on query reply.
- * Called as func(qstate, cb_args)
+ * Called as func(qstate, id, cb_args)
  * Where:
- *	qstate: the query state
- *	id: module id
+ *	qstate: the query state.
+ *	id: module id.
  *	cb_args: argument passed when registering callback.
  */
 typedef int inplace_cb_edns_back_parsed_func_type(struct module_qstate* qstate, 
@@ -287,11 +296,11 @@ typedef int inplace_cb_edns_back_parsed_func_type(struct module_qstate* qstate,
 
 /**
  * Inplace callback function called after parsing query response.
- * Called as func(qstate, id, cb_args)
+ * Called as func(qstate, response, id, cb_args)
  * Where:
- *	qstate: the query state
- *	response: query response
- *	id: module id
+ *	qstate: the query state.
+ *	response: query response.
+ *	id: module id.
  *	cb_args: argument passed when registering callback.
  */
 typedef int inplace_cb_query_response_func_type(struct module_qstate* qstate,
@@ -335,6 +344,8 @@ struct module_env {
 	 * @param zone: delegation point name.
 	 * @param zonelen: length of zone name.
 	 * @param ssl_upstream: use SSL for upstream queries.
+	 * @param tls_auth_name: if ssl_upstream, use this name with TLS
+	 * 	authentication.
 	 * @param q: wich query state to reactivate upon return.
 	 * @return: false on failure (memory or socket related). no query was
 	 *	sent. Or returns an outbound entry with qsent and qstate set.
@@ -345,7 +356,7 @@ struct module_env {
 		uint16_t flags, int dnssec, int want_dnssec, int nocaps,
 		struct sockaddr_storage* addr, socklen_t addrlen,
 		uint8_t* zone, size_t zonelen, int ssl_upstream,
-		struct module_qstate* q);
+		char* tls_auth_name, struct module_qstate* q);
 
 	/**
 	 * Detach-subqueries.
@@ -445,6 +456,10 @@ struct module_env {
 	struct sldns_buffer* scratch_buffer;
 	/** internal data for daemon - worker thread. */
 	struct worker* worker;
+	/** the worker event base */
+	struct comm_base* worker_base;
+	/** the outside network */
+	struct outside_network* outnet;
 	/** mesh area with query state dependencies */
 	struct mesh_area* mesh;
 	/** allocation service */
@@ -468,6 +483,8 @@ struct module_env {
 	struct val_neg_cache* neg_cache;
 	/** the 5011-probe timer (if any) */
 	struct comm_timer* probe_timer;
+	/** auth zones */
+	struct auth_zones* auth_zones;
 	/** Mapping of forwarding zones to targets.
 	 * iterator forwarder information. per-thread, created by worker */
 	struct iter_forwards* fwds;
@@ -610,6 +627,8 @@ struct module_qstate {
 	int no_cache_store;
 	/** whether to refetch a fresh answer on finishing this state*/
 	int need_refetch;
+	/** whether the query (or a subquery) was ratelimited */
+	int was_ratelimited;
 
 	/**
 	 * Attributes of clients that share the qstate that may affect IP-based
@@ -808,4 +827,14 @@ int unique_mesh_state(struct edns_option* list, struct module_env* env);
 void log_edns_known_options(enum verbosity_value level,
 	struct module_env* env);
 
+/**
+ * Copy state that may have happened in the subquery and is always relevant to
+ * the super.
+ * @param qstate: query state that finished.
+ * @param id: module id.
+ * @param super: the qstate to inform.
+ */
+void copy_state_to_super(struct module_qstate* qstate, int id,
+	struct module_qstate* super);
+
 #endif /* UTIL_MODULE_H */
